==> bbperf-0.0.3.tar.gz: bbperf-0.0.3/src/bbperf/util.py <==
# Copyright (c) 2024 Cloudflare, Inc.
# Licensed under the Apache 2.0 license found in the LICENSE file or at https://www.apache.org/licenses/LICENSE-2.0

import socket
import ipaddress

from . import const
from .exceptions import PeerDisconnectedException


def validate_args(args):
    if args.server and args.client:
        raise Exception("ERROR: cannot be both client and server")

    if args.client:
        # this is server IP address
        # allow any exception here to propogate up
        ipaddress.ip_address(args.client)

    if args.port > 65535:
        raise Exception("ERROR: invalid server port {}".format(args.port))

    if args.time <= 0:
        raise Exception("ERROR: argument time must be positive")

    if args.time < 10:
        raise Exception("ERROR: time must be at least 10 seconds")


def convert_bandwidth_str_to_int(arg_str):
    # input string:
    # n[kmgKMG] | n[kmgKMG]pps

    idx = arg_str.find("pps")
    if idx > -1:
        # found, so pps
        is_pps = True
        human_num_str = arg_str[:idx]
    else:
        # bps
        is_pps = False
        human_num_str = arg_str

    last_char = human_num_str[-1:]

    if last_char in [ "k", "K" ]:
        ret_val = int(human_num_str[0:-1]) * (10 ** 3)

    elif last_char in [ "m", "M" ]:
        ret_val = int(human_num_str[0:-1]) * (10 ** 6)

    elif last_char in [ "g", "G" ]:
        ret_val = int(human_num_str[0:-1]) * (10 ** 9)

    else:
        ret_val = int(human_num_str)

    return is_pps, ret_val


def convert_bandwidth_spec_to_batch_info(args, bandwidth_is_pps, bandwidth_val_int):

    if args.udp:
        if bandwidth_is_pps:
            packets_per_sec = bandwidth_val_int
            sends_per_sec = packets_per_sec
        else:
            # udp/bps
            packets_per_sec = (bandwidth_val_int / 8.0) / len(const.PAYLOAD_1K)
            sends_per_sec = packets_per_sec

        batch_size = const.RATE_LIMITED_BATCH_SIZE_PKTS_UDP_PKTS

    else:
        # tcp
        if bandwidth_is_pps:
            packets_per_sec = bandwidth_val_int
            sends_per_sec =  bandwidth_val_int / ( len(const.PAYLOAD_4K) / 1400.0 )
        else:
            packets_per_sec = (bandwidth_val_int / 8.0) / 1400.0
            sends_per_sec = packets_per_sec / ( len(const.PAYLOAD_4K) / 1400.0 )

        batch_size = const.RATE_LIMITED_BATCH_SIZE_PKTS_TCP_PKTS

    batches_per_sec = sends_per_sec / batch_size

    if batches_per_sec < 1:
        batches_per_sec = 1
        batch_size = 1

    delay_between_batches = 1.0 / batches_per_sec

    return batch_size, delay_between_batches


def done_with_socket(mysock):
    try:
        mysock.shutdown(socket.SHUT_RDWR)
    except:
        pass

    try:
        mysock.close()
    except:
        pass


def threads_are_running(thread_list):
    any_running = False

    for t in thread_list:
        if t.is_alive():
            any_running = True

    return any_running


def recv_exact_num_bytes_tcp(client_sock, total_num_bytes_to_read):
    payload_bytes = bytearray()
    num_bytes_read = 0

    while num_bytes_read < total_num_bytes_to_read:

        num_bytes_remaining = total_num_bytes_to_read - num_bytes_read

        # blocking
        recv_bytes = client_sock.recv(num_bytes_remaining)

        num_bytes_received = len(recv_bytes)

        if num_bytes_received == 0:
            raise PeerDisconnectedException()

        num_bytes_read += num_bytes_received

        payload_bytes.extend(recv_bytes)

    return payload_bytes


def parse_r_record(args, s1):
    r_record = {}

    swords = s1.split()

    # literal "a"
    r_record["r_record_type"] = swords[1]
    r_record["r_pkt_sent_time_sec"] = float(swords[2])
    r_record["r_sender_interval_duration_sec"] = float(swords[3])
    r_record["r_sender_interval_pkts_sent"] = int(swords[4])                # valid for udp only
    r_record["r_sender_interval_bytes_sent"] = int(swords[5])
    r_record["r_sender_total_pkts_sent"] = int(swords[6])                   # valid for udp only
    # literal "b"
    r_record["r_receiver_interval_duration_sec"] = float(swords[8])
    r_record["r_receiver_interval_pkts_received"] = int(swords[9])          # valid for udp only
    r_record["r_receiver_interval_bytes_received"] = int(swords[10])
    r_record["r_receiver_total_pkts_received"] = int(swords[11])            # valid for udp only
    # literal "c"
    r_record["r_pkt_received_time_sec"] = float(swords[13])
    # literal "d"

    r_record["rtt_sec"] = r_record["r_pkt_received_time_sec"] - r_record["r_pkt_sent_time_sec"]
    r_record["rtt_ms"] = r_record["rtt_sec"] * 1000

    try:
        # first record received has zeros
        sender_interval_rate_bps = (r_record["r_sender_interval_bytes_sent"] * 8.0) / r_record["r_sender_interval_duration_sec"]
    except ZeroDivisionError:
        sender_interval_rate_bps = 0

    r_record["sender_interval_rate_mbps"] = sender_interval_rate_bps / (10 ** 6)

    r_record["receiver_interval_rate_bytes_per_sec"] = r_record["r_receiver_interval_bytes_received"] / r_record["r_receiver_interval_duration_sec"]

    receiver_interval_rate_bps = r_record["receiver_interval_rate_bytes_per_sec"] * 8
    r_record["receiver_interval_rate_mbps"] = receiver_interval_rate_bps / (10 ** 6)

    r_record["buffered_bytes"] = int( r_record["receiver_interval_rate_bytes_per_sec"] * r_record["rtt_sec"] )

    if args.udp:
        try:
            # first record received has zeroes
            r_record["sender_pps"] = int(r_record["r_sender_interval_pkts_sent"] / r_record["r_sender_interval_duration_sec"])
        except ZeroDivisionError:
            r_record["sender_pps"] = 0

        r_record["receiver_pps"] = int(r_record["r_receiver_interval_pkts_received"] / r_record["r_receiver_interval_duration_sec"])
        r_record["total_dropped"] = r_record["r_sender_total_pkts_sent"] - r_record["r_receiver_total_pkts_received"]

        assert r_record["total_dropped"] >= 0
    else:
        r_record["sender_pps"] = -1
        r_record["receiver_pps"] = -1
        r_record["total_dropped"] = -1

    return r_record

==> bbperf-0.0.6.tar.gz: bbperf-0.0.6/src/bbperf/util.py <==
# Copyright (c) 2024 Cloudflare, Inc.
# Licensed under the Apache 2.0 license found in the LICENSE file or at https://www.apache.org/licenses/LICENSE-2.0

import sys
import socket
import ipaddress

from . import const
from .exceptions import PeerDisconnectedException


def validate_args(args):
    if args.server and args.client:
        raise Exception("ERROR: cannot be both client and server")

    if args.client:
        # this is server IP address
        # allow any exception here to propogate up
        ipaddress.ip_address(args.client)

    if args.port > 65535:
        raise Exception("ERROR: invalid server port {}".format(args.port))

    if args.verbosity and args.quiet:
        raise Exception("ERROR: cannot specify both verbosity and quiet")


def convert_udp_pps_to_batch_info(packets_per_sec):

    batch_size = const.RATE_LIMITED_BATCH_SIZE_PKTS_UDP_PKTS

    batches_per_sec = packets_per_sec / batch_size

    if batches_per_sec < 1:
        batches_per_sec = 1
        batch_size = 1

    delay_between_batches = 1.0 / batches_per_sec

    return batch_size, delay_between_batches


def done_with_socket(mysock):
    try:
        mysock.shutdown(socket.SHUT_RDWR)
    except:
        pass

    try:
        mysock.close()
    except:
        pass


def threads_are_running(thread_list):
    any_running = False

    for t in thread_list:
        if t.is_alive():
            any_running = True

    return any_running


def recv_exact_num_bytes_tcp(client_sock, total_num_bytes_to_read):
    payload_bytes = bytearray()
    num_bytes_read = 0

    while num_bytes_read < total_num_bytes_to_read:

        num_bytes_remaining = total_num_bytes_to_read - num_bytes_read

        # blocking
        recv_bytes = client_sock.recv(num_bytes_remaining)

        num_bytes_received = len(recv_bytes)

        if num_bytes_received == 0:
            raise PeerDisconnectedException()

        num_bytes_read += num_bytes_received

        payload_bytes.extend(recv_bytes)

    return payload_bytes


def parse_r_record(args, s1):
    r_record = {}

    swords = s1.split()

    # literal "a"
    r_record["r_record_type"] = swords[1]
    r_record["r_pkt_sent_time_sec"] = float(swords[2])
    r_record["r_sender_interval_duration_sec"] = float(swords[3])
    r_record["r_sender_interval_pkts_sent"] = int(swords[4])                # valid for udp only
    r_record["r_sender_interval_bytes_sent"] = int(swords[5])
    r_record["r_sender_total_pkts_sent"] = int(swords[6])                   # valid for udp only
    # literal "b"
    r_record["r_receiver_interval_duration_sec"] = float(swords[8])
    r_record["r_receiver_interval_pkts_received"] = int(swords[9])          # valid for udp only
    r_record["r_receiver_interval_bytes_received"] = int(swords[10])
    r_record["r_receiver_total_pkts_received"] = int(swords[11])            # valid for udp only
    # literal "c"
    r_record["r_pkt_received_time_sec"] = float(swords[13])
    r_record["interval_dropped"] = int(swords[14])
    r_record["interval_dropped_percent"] = float(swords[15])
    r_record["is_sample_valid"] = int(swords[16])
    # literal "d"

    r_record["rtt_sec"] = r_record["r_pkt_received_time_sec"] - r_record["r_pkt_sent_time_sec"]
    r_record["rtt_ms"] = r_record["rtt_sec"] * 1000

    try:
        # first record received has zeros
        sender_interval_rate_bps = (r_record["r_sender_interval_bytes_sent"] * 8.0) / r_record["r_sender_interval_duration_sec"]
    except ZeroDivisionError:
        sender_interval_rate_bps = 0

    r_record["sender_interval_rate_mbps"] = sender_interval_rate_bps / (10 ** 6)

    r_record["receiver_interval_rate_bytes_per_sec"] = r_record["r_receiver_interval_bytes_received"] / r_record["r_receiver_interval_duration_sec"]

    receiver_interval_rate_bps = r_record["receiver_interval_rate_bytes_per_sec"] * 8
    r_record["receiver_interval_rate_mbps"] = receiver_interval_rate_bps / (10 ** 6)

    r_record["buffered_bytes"] = int( r_record["receiver_interval_rate_bytes_per_sec"] * r_record["rtt_sec"] )

    if args.udp:
        try:
            # first record received has zeroes
            r_record["sender_pps"] = int(r_record["r_sender_interval_pkts_sent"] / r_record["r_sender_interval_duration_sec"])
        except ZeroDivisionError:
            r_record["sender_pps"] = 0

        r_record["receiver_pps"] = int(r_record["r_receiver_interval_pkts_received"] / r_record["r_receiver_interval_duration_sec"])
        r_record["total_dropped"] = r_record["r_sender_total_pkts_sent"] - r_record["r_receiver_total_pkts_received"]

        if r_record["total_dropped"] < 0:
            print("ERROR: total dropped is less than zero: sent {} received {}".format(
                r_record["r_sender_total_pkts_sent"],
                r_record["r_receiver_total_pkts_received"]),
                file=sys.stderr)

    else:
        r_record["sender_pps"] = -1
        r_record["receiver_pps"] = -1
        r_record["total_dropped"] = -1

    return r_record
